# -*- coding: utf-8 -*-
"""
Base method to parse title
"""
import re
from functools import reduce

SEPARATORS = [
    ' -- ',
    '--',
    ' - ',
    ' – ',
    ' — ',
    ' _ ',
    '-',
    '–',
    '—',
    ':',
    '|',
    '///',
    ' / ',
    '_',
    '/'
]

def clean_mvpv(text):
    """
    Remove various versions of "MV" and "PV" markers
    """
    text = re.sub(
        r'\s*\[\s*(?:off?icial\s+)?([PM]\/?V)\s*]', '', text, flags=re.IGNORECASE) # [MV] or [M/V]
    text = re.sub(
        r'\s*\(\s*(?:off?icial\s+)?([PM]\/?V)\s*\)', '', text, flags=re.IGNORECASE) # (MV) or (M/V)
    text = re.sub(
        r'\s*【\s*(?:off?icial\s+)?([PM]\/?V)\s*】', '', text, flags=re.IGNORECASE) # 【MV】 or 【M/V】
    text = re.sub(
        r'[\s\-–_]+(?:off?icial\s+)?([PM]\/?V)\s*',
        '', text, flags=re.IGNORECASE) # MV or M/V at the end
    text = re.sub(r'(?:off?icial\s+)?([PM]\/?V)[\s\-–_]+', '', text) # MV or M/V at the start
    return text

def clean_fluff(text):
    """
    Remove fluff
    """
    text = clean_mvpv(text)
    text = re.sub(r'\s*\[[^\]]+]$', '', text) # [whatever] at the end
    text = re.sub(r'^\s*\[[^\]]+]\s*', '', text) # [whatever] at the start
    text = re.sub(
        r'\s*\([^)]*\bver(\.|sion)?\s*\)$', '', text, flags=re.IGNORECASE) # (whatever version)
    text = re.sub(
        r'\s*[a-z]*\s*\bver(\.|sion)?$',
        '', text, flags=re.IGNORECASE) # ver. & 1 word before (no parens)
    text = re.sub(
        r'\s*(of+icial\s*)?(music\s*)?video',
        '', text, flags=re.IGNORECASE) # (official)? (music)? video
    text = re.sub(
        r'\s*(ALBUM TRACK\s*)?(album track\s*)', '', text, flags=re.IGNORECASE) # (ALBUM TRACK)
    text = re.sub(r'\s*\(\s*of+icial\s*\)', '', text, flags=re.IGNORECASE) # (official)
    text = re.sub(r'\s*\(\s*lyric(s)?\s*\)', '', text, flags=re.IGNORECASE) # (lyrics)
    text = re.sub(
        r'\s*\(\s*(of+icial)?\s*lyric(s)?\s*\)', '', text, flags=re.IGNORECASE) # (official lyrics)
    text = re.sub(r'\s*\(\s*[0-9]{4}\s*\)', '', text, flags=re.IGNORECASE) # (1999)
    text = re.sub(r'\s+\(\s*(HD|HQ)\s*\)$', '', text) # HD (HQ)
    text = re.sub(r'[\s\-–_]+(HD|HQ)\s*$', '', text) # HD (HQ)
    return text

def clean_title(title):
    """
    Clean song title
    """
    title = title.strip(' ')
    title = clean_fluff(title)
    title = re.sub(r'\s*\*+\s?\S+\s?\*+$', '', title) # **NEW**
    title = re.sub(r'\s*video\s*clip', '', title, flags=re.IGNORECASE) # video clip
    title = re.sub(r'\s+\(?live\)?$', '', title, flags=re.IGNORECASE) # live
    title = re.sub(r'\(\s*\)', '', title) # Leftovers after e.g. (official video)
    title = re.sub(r'\[\s*]', '', title) # Leftovers after e.g. [1080p]
    title = re.sub(r'【\s*】', '', title) # Leftovers after e.g. 【MV】
    title = re.sub(
        r'^(|.*\s)"(.*)"(\s.*|)$', r'\3', title) # Artist - The new "Track title" featuring someone
    title = re.sub(r"^(|.*\s)'(.*)'(\s.*|)$", r'\3', title) # 'Track title'
    title = re.sub(r'^[/\s,:;~\-–_\s"]+', '', title) # trim starting white chars and dash
    title = re.sub(r'[/\s,:;~\-–_\s"]+$', '', title) # trim trailing white chars and dash
    return title

def clean_artist(artist):
    """
    Clean artist name
    """
    artist = artist.strip(' ')
    artist = clean_fluff(artist)
    artist = re.sub(r'\s*[0-1][0-9][0-1][0-9][0-3][0-9]\s*', '', artist) # date formats ex. 130624
    artist = re.sub(r'^[/\s,:;~\-–_\s"]+', '', artist) # trim starting white chars and dash
    artist = re.sub(r'[/\s,:;~\-–_\s"]+$', '', artist) # trim trailing white chars and dash
    return artist

def in_quotes(text, idx):
    open_chars = '([{«'
    close_chars = ')]}»'
    toggle_chars = '"\''
    open_pars = {
        ')': 0,
        ']': 0,
        '}': 0,
        '»': 0,
        '"': 0,
        '\'': 0
    }
    for character in text[:idx]:
        index = open_chars.find(character)
        if index != -1:
            open_pars[close_chars[index]] += 1
        elif close_chars.find(character) != -1 and open_pars[character] > 0:
            open_pars[character] -= 1
        if toggle_chars.find(character) != -1:
            open_pars[character] = 1 - open_pars[character]
    return reduce((lambda acc, value: acc + value), open_pars.values()) > 0

def split_artist_title(text):
    """
    Split text at separators
    """
    for separator in SEPARATORS:
        try:
            idx = text.index(separator)
        except ValueError:
            continue
        if idx > -1 and not in_quotes(text, idx):
            return [text[:idx], text[idx + len(separator):]]



# -*- coding: utf-8 -*-
"""
Parse the title of a YouTube video to try and get artist & song name
"""
from __future__ import print_function, absolute_import
import argparse
try:
    import youtube_title_parse.plugins as plugins
    from youtube_title_parse.core import mapArtistTitle, mapTitle, get_song_artist_title
except ImportError:
    import plugins
    from core import mapArtistTitle, mapTitle, get_song_artist_title

def get_artist_title(text, options={}):
    """
    Parse method
    """
    return get_song_artist_title(text, options, {
        'before': [plugins.remove_file_extensions, plugins.clean_fluff],
        'split': [plugins.split_artist_title, plugins.split_text],
        'after': [mapArtistTitle(plugins.clean_artist, plugins.clean_title),
                  mapArtistTitle(plugins.clean, plugins.clean), mapTitle(plugins.clean_common_fluff)]
    })

def process(args):
    options = {}
    if args.defaultArtist:
        options['defaultArtist'] = args.defaultArtist
    if args.defaultTitle:
        options['defaultTitle'] = args.defaultTitle
    result = get_artist_title(args.youtube_title, options)
    if result:
        print("%s - %s" % (result[0], result[1]))
    else:
        print(args.youtube_title)
    return result

def main():
    argparser = argparse.ArgumentParser(
        description='youtube_title_parse', formatter_class=argparse.RawTextHelpFormatter)
    argparser.add_argument(
        "youtube_title", type=str, help='required youtube video title')
    argparser.add_argument(
        "-da", "--defaultArtist", dest="defaultArtist", type=str,
        metavar='DEFAULT_ARTIST', default=False,
        help="""
        fallback artist name
        This value will be used as the 'artist' if
        parser couldn't get artist from YouTube title.
        """
        )
    argparser.add_argument(
        "-dt", "--defaultTitle", dest="defaultTitle", type=str,
        metavar='DEFAULT_TITLE', default=False,
        help="""
        fallback song title
        This value will be used as the 'title' if
        parser couldn't get song title from YouTube title.
        """
        )
    args = argparser.parse_args()
    process(args)

if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
"""
Remove file extensions from string
"""
import re

VIDEO_EXTENSIONS = ["3g2", "3gp", "aaf", "asf", "avchd", "avi",
                    "drc", "flv", "m2v", "m4p", "m4v", "mkv", "mng", "mov", "mp2",
                    "mp4", "mpe", "mpeg", "mpg", "mpv", "mxf", "nsv", "ogg", "ogv",
                    "qt", "rm", "rmvb", "roq", "svi", "vob", "webm", "wmv", "yuv"]

AUDIO_EXTENSIONS = ["wav", "bwf", "raw", "aiff", "flac", "m4a",
                    "pac", "tta", "wv", "ast", "aac", "mp2", "mp3", "mp4", "amr",
                    "s3m", "3gp", "act", "au", "dct", "dss", "gsm", "m4p", "mmf",
                    "mpc", "ogg", "oga", "opus", "ra", "sln", "vox"]

FILE_EXTENSIONS = VIDEO_EXTENSIONS + AUDIO_EXTENSIONS
FILE_EXTENSIONS_RX = re.compile(r'\.(' + '|'.join(FILE_EXTENSIONS) + ')$', re.IGNORECASE)

def remove_file_extensions(title):
    return FILE_EXTENSIONS_RX.sub('', title)



# -*- coding: utf-8 -*-
"""
Core parsing methods
"""
try:
    from youtube_title_parse.fallback_artist import fallback_artist
    from youtube_title_parse.fallback_title import fallback_title
except ImportError:
    from fallback_artist import fallback_artist
    from fallback_title import fallback_title

def flow(functions):
    if not functions:
        def failed(arg):
            return arg
        return failed

    def flow_func(arguments):
        result = functions[0](arguments)
        for function in functions[1:]:
            result = function(result)
        return result
    return flow_func

def combine_splitters(splitters):
    def combine_func(text):
        for splitter in splitters:
            result = splitter(text)
            if result:
                return result
    return combine_func

def reduce_plugins(plugins):
    return [flow(plugins['before']), combine_splitters(plugins['split']), flow(plugins['after'])]

def checkPlugin(plugin):
    if not plugin[1]:
        print('no title splitter was specified by any plugin')

def mapArtist(fn):
    def mapA(parts):
        return [fn(parts[0]), parts[1]]
    return mapA

def mapTitle(fn):
    def mapT(parts):
        return [parts[0], fn(parts[1])]
    return mapT

def mapArtistTitle(map_artist, map_title):
    def mapAT(parts):
        return [map_artist(parts[0]), map_title(parts[1])]
    return mapAT

def get_song_artist_title(text, options, plugins):
    if options:
        if 'defaultArtist' in options:
            plugins['split'].append(fallback_artist(options['defaultArtist']))
        if 'defaultTitle' in options:
            plugins['split'].append(fallback_title(options['defaultTitle']))
    plugin = reduce_plugins(plugins)
    checkPlugin(plugin)

    split = plugin[1](plugin[0](text))
    if not split:
        return
    return plugin[2](split)



# -*- coding: utf-8 -*-
"""
Handle titles with quotes
"""
import re
from functools import reduce

QUOTES = [
    '“”',
    '""',
    '\'\''
]

def looseRegs(sset):
    open_set = sset[0]
    close_set = sset[1]
    return re.compile(open_set + r'(.*?)' + close_set)
MATCH_LOOSE_RXES = map(looseRegs, QUOTES)

def startRegs(sset):
    open_set = sset[0]
    close_set = sset[1]
    return re.compile(r'^' + open_set + r'(.*?)' + close_set + r'\\s*')
MATCH_START_RXES = map(startRegs, QUOTES)

def split_text(text):
    for loose_rex in MATCH_LOOSE_RXES:
        text = re.sub(loose_rex, ' ' + r'\1' + ' ', text)
        match = re.search(loose_rex, text)
        if match:
            split = match.start()
            title = text[split:]
            artist = text[:, split]
            return [artist, title]

def clean(artistOrTitle):
    return reduce((lambda text, rx: re.sub(
        rx, r'\1 ', text)), MATCH_START_RXES, artistOrTitle).strip()



"""
Setup script for youtube_title_parse
"""
from setuptools import setup, find_packages

setup(
    name='youtube_title_parse',
    description='Parse the title of a YouTube video to try and get artist & song name',
    version='1.0.0b1',
    url='https://github.com/lttkgp/youtube-title-parse',
    author='Naresh R',
    author_email='ghostwriternr@gmail.com',
    license='GNU General Public License v3 (GPLv3)',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Natural Language :: English',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],
    keywords=['youtube', 'title', 'metadata', 'parse', 'music'],
    packages=find_packages(exclude=['contrib', 'docs', 'tests*']),
    install_requires=[],
    entry_points={
        'console_scripts': [
            'youtube_title_parse=youtube_title_parse.parse:main'
        ],
    }
)
